#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <vector>
#include <algorithm>

using namespace std;

// Size of the map, radius of each neon light
long long N, M, R;
// Cost of one light, cost of one turn, budget
long long C, P, B;
// The map
vector<string> Map;

// The committee thresholds
long long Lo, Hi;

// Competitor's list of neon lights' positions
vector<pair<long long, long long>> Neons;
// Map positions back to neons in these positions
int Inv[1001][1001];

/****** DSU START ******/

long long numComponents;

struct Node
{
    int parent;
    int rank;
};
vector<Node> DSU;

inline void MakeSet(int x)
{
    DSU[x].parent = x;
    DSU[x].rank = 0;
}

inline void InitDSU(int tot)
{
    numComponents = tot;
    DSU.resize(tot + 1);
    for (int i=1;i<=tot;i++) MakeSet(i);
}

inline int Find(int x)
{
    if (DSU[x].parent == x) return x;
    DSU[x].parent = Find(DSU[x].parent);
    return DSU[x].parent;
}

inline void Union(int x, int y)
{
    int xRoot = Find(x);
    int yRoot = Find(y);
    if (xRoot == yRoot) return;
    numComponents--;
    if (DSU[xRoot].rank < DSU[yRoot].rank)
    {
        DSU[xRoot].parent = yRoot;
    }
    else if (DSU[xRoot].rank > DSU[yRoot].rank)
    {
        DSU[yRoot].parent = xRoot;
    }
    else
    {
        DSU[yRoot].parent = xRoot;
        DSU[xRoot].rank++;
    }
}

/****** DSU END ******/

/*
 This method will return two integers:
    - The number of fields caught in the blast
    - The number of connected components
*/
long long get_value()
{
    // Initialise the DSU
    InitDSU(Neons.size());

    // Store which neon illuminated a position first (for efficient merging)
    bool mark[N][M];
    for (int i=0;i<N;i++)
    {
        for (int j=0;j<M;j++)
        {
            mark[i][j] = false;
        }
    }

    long long total = 0;

    for (int i=0;i<Neons.size();i++)
    {
        // Out of bounds etc was already checked in main, so can proceed straight to flooding

        int ly = -1;
        int ry = M;

        for (int dx=0;dx>=-R;dx--)
        {
            int xt = Neons[i].first + dx;
            if (xt < 0) break;
            if (Map[xt][Neons[i].second] != '.') break;

            // top-left
            for (int dy=0;dy>=-R;dy--)
            {
                int yt = Neons[i].second + dy;
                if (yt <= ly) break;
                if (Map[xt][yt] != '.')
                {
                    ly = max(ly, yt);
                    break; // no need to go on
                }

                // Always combine any unconnected components
                Union(i + 1, Inv[xt][yt]);

                // If field is unoccupied, occupy it
                if (!mark[xt][yt])
                {
                    mark[xt][yt] = true;
                    total++;
                }
            }

            // top-right
            for (int dy=0;dy<=R;dy++)
            {
                int yt = Neons[i].second + dy;
                if (yt >= ry) break;
                if (Map[xt][yt] != '.')
                {
                    ry = min(ry, yt);
                    break;
                }

                Union(i + 1, Inv[xt][yt]);

                if (!mark[xt][yt])
                {
                    mark[xt][yt] = true;
                    total++;
                }
            }
        }

        ly = -1;
        ry = M;

        // now expand down
        for (int dx=0;dx<=R;dx++)
        {
            int xt = Neons[i].first + dx;
            if (xt > N - 1) break;
            if (Map[xt][Neons[i].second] != '.') break;

            // bottom-left
            for (int dy=0;dy>=-R;dy--)
            {
                int yt = Neons[i].second + dy;
                if (yt <= ly) break;
                if (Map[xt][yt] != '.')
                {
                    ly = max(ly, yt);
                    break; // no need to go on
                }

                Union(i + 1, Inv[xt][yt]);

                if (!mark[xt][yt])
                {
                    mark[xt][yt] = true;
                    total++;
                }
            }

            // top-right
            for (int dy=0;dy<=R;dy++)
            {
                int yt = Neons[i].second + dy;
                if (yt >= ry) break;
                if (Map[xt][yt] != '.')
                {
                    ry = min(ry, yt);
                    break;
                }

                Union(i + 1, Inv[xt][yt]);

                if (!mark[xt][yt])
                {
                    mark[xt][yt] = true;
                    total++;
                }
            }
        }
    }

    return total;
}

int main(int argc, char *argv[])
{
    ifstream outFile(argv[1]);
    ifstream solFile(argv[2]);
    ifstream inFile(argv[3]); 

    // Read in the size and radius
    inFile >> N >> M >> R;
    // Read in the cost parameters
    inFile >> C >> P >> B;

    // Read in the map
    Map.resize(N);
    for (int i=0;i<N;i++)
    {
        // Read in current line of the map
        inFile >> Map[i];
    }

    // Read in the committee thresholds
    solFile >> Lo >> Hi;

    Neons.clear();
    // Keep track of contestant's total cost
    long long total_cost = 0LL;

    int Nid = 0;
    long long Nx, Ny;
    // Read in all the neons
    while (outFile >> Nx)
    {
        ++Nid;
        if (outFile >> Ny)
        {
            // Convert the neon position to zero-indexed
            Nx--; Ny--;

            // Check if competitor exceeded limits
            if (Nx < 0 || Nx > N - 1 || Ny < 0 || Ny > M - 1)
            {
                return -1;
            }

            // Check if competitor hit wall
            if (Map[Nx][Ny] != '.')
            {
                return -1;
            }

            // CHeck if neon already there
            if (Inv[Nx][Ny])
            {
                return -1;
            }

            // Check if competitor exceeded cost
            total_cost += C;
            if (total_cost > B)
            {
                return -1;
            }

            // If all is well, push this neon with the rest
            Neons.push_back({Nx, Ny});
            Inv[Nx][Ny] = Nid;
        }
        else
        {
            // Competitor entered an odd number of inputs!
            return -1;
        }
    }

    // Obtain value and (incidentally) set numComponents
    long long ret = get_value();
    // Add up to total cost value
    total_cost += numComponents * P;
    // Check if competitor exceeded cost
    if (total_cost > B)
    {
        return -1;
    }

    outFile.close();
    solFile.close();
    inFile.close();

    // Compute points and gracefully finish :)
    if (ret <= Lo) return 0;
    else if (ret >= Hi) return 25;
    else return (25 * (ret - Lo)) / (Hi - Lo);
}